﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Syncfusion.DocIO;
using Syncfusion.DocIO.DLS;
using Syncfusion.DocIORenderer;
using Syncfusion.OfficeChart;
using Syncfusion.Pdf;

namespace DocIOProblem.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class ReportsController : ControllerBase
    {
        private IHostingEnvironment _env;

        public ReportsController(IHostingEnvironment env)
        {
            _env = env;
        }


        //[HttpGet("[action]")]
        //public IActionResult CreateDoc()
        //{
        //    string file = _env.ContentRootPath + @"\template\order.docx";
        //    MemoryStream stream = GenerateNewREport(file);

        //    return File(stream, "application/msword", "Result.docx");
        //}

        [HttpGet("[action]")]
        public IActionResult CreateDocError()
        {
            string file = _env.ContentRootPath + @"\template\order_error.docx";
            try
            {
                MemoryStream stream = GenerateNewREport(file);
                var streamPdf = ConvertPdf(stream);
                //return File(stream, "application/msword", "Result.docx");
                return File(streamPdf, "application/pdf", "ResumenOferta.pdf");
            }
            catch (Exception ex)
            {

                return BadRequest(ex.Message);
            }
        }

        private MemoryStream ConvertPdf(MemoryStream stream)
        {
            WordDocument wordDocument = new WordDocument(stream, Syncfusion.DocIO.FormatType.Automatic);
            DocIORenderer render = new DocIORenderer();
            render.Settings.ChartRenderingOptions.ImageFormat = ExportImageFormat.Jpeg;

            render.Settings.EmbedCompleteFonts = true;
            //render.Settings.AutoTag = true;
            render.Settings.OptimizeIdenticalImages = true;
            // render.Settings.PdfConformanceLevel = PdfConformanceLevel.Pdf_A1B;

            PdfDocument pdfDocument = render.ConvertToPDF(wordDocument);

            render.Dispose();
            wordDocument.Dispose();

            MemoryStream outputStream = new MemoryStream();
            pdfDocument.Save(outputStream);
            //Closes the instance of PDF document object
            pdfDocument.Close();
            outputStream.Position = 0;
            return outputStream;
        }

        private static MemoryStream GenerateNewREport(string file)
        {
            MemoryStream stream;

            var fileStreamDoc = new FileStream(file, FileMode.Open);

            using (WordDocument document = new WordDocument(fileStreamDoc, FormatType.Docx))
            {
                WordDocument clonedDocument = document.Clone();
                fileStreamDoc.Close();

                // 
                clonedDocument.Replace("dbNumeroSolicitud", "1", true, false);
                clonedDocument.Replace("dbFechaGeneracionOrden", DateTime.Now.ToString("dd/MM/yyyy"), true, false);

                TextSelection textSelection = clonedDocument.Find("dbDescripcionResumen", false, true);
                WTextRange txt = textSelection.GetAsOneRange();
                txt.Text = "data1\n";
                txt.Text += "data2\n";
                txt.Text += "data3\n";
                txt.Text += "data4\n";
                WParagraph ownerParagraph = txt.Owner as WParagraph;
                //Applies default numbered list style
                ownerParagraph.ListFormat.ApplyDefBulletStyle();

                clonedDocument.Replace("dbFechaOferta", DateTime.Now.ToString("dd/MM/yyyy"), true, false);

                clonedDocument.Replace("dbContado ", "1432", true, false);
                clonedDocument.Replace("db12", "1432", true, false);
                clonedDocument.Replace("db36", "1432", true, false);
                clonedDocument.Replace("db60", "1432", true, false);

                stream = new MemoryStream();
                //Saves the Word document to  MemoryStream
                clonedDocument.Save(stream, FormatType.Docx);
                clonedDocument.Close();
                stream.Position = 0;
            }

            return stream;
        }
    }
}